@@ -6,10 +6,10 @@ use 5.010;
 
 use parent 'Class::Accessor';
 
-our $VERSION = '0.05';
+our $VERSION = '1.01';
 
 Travel::Status::DE::DeutscheBahn::Result->mk_ro_accessors(
-	qw(time train route_end route_raw platform info_raw));
+	qw(date time train route_end route_raw platform info_raw));
 
 sub new {
 	my ( $obj, %conf ) = @_;
@@ -25,6 +25,12 @@ sub destination {
 	return $self->{route_end};
 }
 
+sub line {
+	my ($self) = @_;
+
+	return $self->{train};
+}
+
 sub info {
 	my ($self) = @_;
 
@@ -32,7 +38,8 @@ sub info {
 
 	$info =~ s{ ,Grund }{}ox;
 	$info =~ s{ ^ \s+ }{}ox;
-	$info =~ s{ (?: ^ | , ) (?: p.nktlich | k [.] A [.] ) }{}ox;
+	$info
+	  =~ s{ (?: ^ | , ) (?: p.nktlich | [nk] [.] [Aa] [.] | on \s time ) }{}ox;
 	$info =~ s{ ^ , }{}ox;
 
 	return $info;
@@ -62,7 +69,8 @@ sub origin {
 sub route {
 	my ($self) = @_;
 
-	return @{ $self->{route} };
+	my @stops = map { $_->[1] } @{ $self->{route} };
+	return @stops;
 }
 
 sub route_interesting {
@@ -115,6 +123,12 @@ sub route_interesting {
 
 }
 
+sub route_timetable {
+	my ($self) = @_;
+
+	return @{ $self->{route} };
+}
+
 1;
 
 __END__
@@ -130,7 +144,7 @@ arrival/departure received by Travel::Status::DE::DeutscheBahn
 		printf(
 			"At %s: %s to %s from platform %s\n",
 			$departure->time,
-			$departure->train,
+			$departure->line,
 			$departure->destination,
 			$departure->platform,
 		);
@@ -141,7 +155,7 @@ arrival/departure received by Travel::Status::DE::DeutscheBahn
 		printf(
 			"At %s: %s from %s on platform %s\n",
 			$arrival->time,
-			$arrival->train,
+			$arrival->line,
 			$arrival->origin,
 			$arrival->platform,
 		);
@@ -149,7 +163,7 @@ arrival/departure received by Travel::Status::DE::DeutscheBahn
 
 =head1 VERSION
 
-version 0.05
+version 1.01
 
 =head1 DESCRIPTION
 
@@ -163,6 +177,36 @@ the platform, time, route and more.
 
 =over
 
+=item $result->date
+
+Arrival/Departure date in "dd.mm.yyyy" format.
+
+=item $result->delay
+
+Returns the train's delay in minutes, or undef if it is unknown.
+
+=item $result->info
+
+Returns additional information, for instance the reason why the train is
+delayed. May be an empty string if no (useful) information is available.
+
+=item $result->line
+
+=item $result->train
+
+Returns the line name, either in a format like "S 1" (S-Bahn line 1)
+or "RE 10111" (RegionalExpress train 10111, no line information).
+
+=item $result->platform
+
+Returns the platform from which the train will depart / at which it will
+arrive.
+
+=item $result->route
+
+Returns a list of station names the train will pass between the selected
+station and its origin/destination.
+
 =item $result->route_end
 
 Returns the last element of the route.  Depending on how you set up
@@ -175,25 +219,6 @@ either the train's destination or its origin station.
 
 Convenience aliases for $result->route_end.
 
-=item $result->delay
-
-Returns the train's delay in minutes, or undef if it is unknown.
-
-=item $result->info
-
-Returns additional information, for instance the reason why the train is
-delayed. May be an empty string if no (useful) information is available.
-
-=item $result->platform
-
-Returns the platform from which the train will depart / at which it will
-arrive.
-
-=item $result->route
-
-Returns a list of station names the train will pass between the selected
-station and its origin/destination.
-
 =item $result->route_interesting([I<max>])
 
 Returns a list of up to I<max> (default: 3) interesting stations the train
@@ -217,15 +242,15 @@ Returns the raw string used to create the route array.
 Note that canceled stops are filtered from B<route>, but still present in
 B<route_raw>.
 
+=item $result->route_timetable
+
+Similar to B<route>.  however, this function returns a list of array
+references of the form C<< [ arrival time, station name ] >>.
+
 =item $result->time
 
 Returns the arrival/departure time as string in "hh:mm" format.
 
-=item $result->train
-
-Returns the line / train name, either in a format like "S 1" (S-Bahn line 1)
-or "RE 10111" (RegionalExpress train 10111, no line information).
-
 =back
 
 =head2 INTERNAL
@@ -273,7 +298,7 @@ None.
 
 =head1 BUGS AND LIMITATIONS
 
-Arrival times are present in B<route_raw>, but not accessible via B<route>.
+None known.
 
 =head1 SEE ALSO
 
